import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.Random;
public class FrameAircraftCarrier extends JFrame {
    private DrawingAircraft drawingAircraft;
    private AbstractStrategy abstractStrategy;
    private JComboBox comboBoxStrategy;
    private final JComponent pictureBox;
    public FrameAircraftCarrier() throws IOException {
        super("Военный корабль");
        setSize(new Dimension(900,500));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pictureBox = new JComponent(){
            public void paintComponent(Graphics graphics){
                super.paintComponent(graphics);
                Graphics2D graphics2D = (Graphics2D) graphics;
                if (drawingAircraft!= null) drawingAircraft.drawTransport(graphics2D);
                super.repaint();
            }
        };
        comboBoxStrategy = new JComboBox<>(new String[]{"К центру", "К границе"});
        JButton stepButton = new JButton("Шаг");
        JButton createShipButton = new JButton("Создать корабль");
        JButton createAircraftCarrierButton = new JButton("Создать Военный корабль");
        JButton rightButton = new JButton(new ImageIcon(ImageIO.read(new File("C:\\Users\\79962\\Desktop\\arrows\\right.png"))));
        JButton leftButton = new JButton(new ImageIcon(ImageIO.read(new File("C:\\Users\\79962\\Desktop\\arrows\\left.png"))));
        JButton upButton = new JButton(new ImageIcon(ImageIO.read(new File("C:\\Users\\79962\\Desktop\\arrows\\up.png"))));
        JButton downButton = new JButton(new ImageIcon(ImageIO.read(new File("C:\\Users\\79962\\Desktop\\arrows\\down.png"))));
        pictureBox.setBounds( 0, 0, getContentPane().getWidth(), getContentPane().getHeight());
        createShipButton.addActionListener(e -> buttonCreateShipClick());
        createAircraftCarrierButton.addActionListener(e -> buttonCreateAircraftCarrierClick());
        stepButton.addActionListener(e -> buttonStepClick());
        rightButton.setActionCommand("right");
        rightButton.addActionListener(this::buttonMoveClick);
        leftButton.setActionCommand("left");
        leftButton.addActionListener(this::buttonMoveClick);
        upButton.setActionCommand("up");
        upButton.addActionListener(this::buttonMoveClick);
        downButton.setActionCommand("down");
        downButton.addActionListener(this::buttonMoveClick);
        //component addition
        setLayout(new BorderLayout());
        JPanel panelAircraftCarrier = new JPanel(new BorderLayout());
        JPanel rightPanel = new JPanel(new BorderLayout());
        JPanel leftPanel = new JPanel(new BorderLayout());
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets.left = constraints.insets.top = constraints.insets.bottom = constraints.insets.right = 2;
        JPanel createPanel = new JPanel(new GridBagLayout());
        constraints.gridx = 0;
        constraints.gridy = 0;
        createPanel.add(createShipButton, constraints);
        constraints.gridx = 1;
        constraints.gridy = 0;
        createPanel.add(createAircraftCarrierButton, constraints);
        JPanel movementPanel = new JPanel(new GridBagLayout());
        rightButton.setPreferredSize(new Dimension(30,30));
        constraints.gridx = 2;
        constraints.gridy = 1;
        movementPanel.add(rightButton, constraints);
        leftButton.setPreferredSize(new Dimension(30,30));
        constraints.gridx = 0;
        constraints.gridy = 1;
        movementPanel.add(leftButton, constraints);
        upButton.setPreferredSize(new Dimension(30,30));
        constraints.gridx = 1;
        constraints.gridy = 0;
        movementPanel.add(upButton, constraints);
        downButton.setPreferredSize(new Dimension(30,30));
        constraints.gridx = 1;
        constraints.gridy = 1;
        movementPanel.add(downButton, constraints);
        JPanel stepPanel = new JPanel(new GridBagLayout());
        constraints.gridx = 0;
        constraints.gridy = 0;
        stepPanel.add(comboBoxStrategy, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        stepPanel.add(stepButton, constraints);
        add(pictureBox);
        rightPanel.add(movementPanel, BorderLayout.SOUTH);
        rightPanel.add(stepPanel, BorderLayout.NORTH);
        leftPanel.add(createPanel, BorderLayout.SOUTH);
        panelAircraftCarrier.add(rightPanel, BorderLayout.EAST);
        panelAircraftCarrier.add(leftPanel, BorderLayout.WEST);
        add(panelAircraftCarrier,BorderLayout.CENTER);
        setVisible(true);
    }
    private void buttonCreateAircraftCarrierClick() {
        Random random = new Random();
        pictureBox.setBounds(0,0,getContentPane().getWidth(),getContentPane().getHeight());
        drawingAircraft = new DrawingAircraftCarrier(random.nextInt(200) + 100, random.nextInt(2000) + 1000, new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)),
                new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)), random.nextBoolean(), random.nextBoolean(), pictureBox.getWidth(), pictureBox.getHeight(), random.nextInt(3),(random.nextInt(3)+1)*2);
        drawingAircraft.setPosition(random.nextInt(90) + 10, random.nextInt(90) + 10);
        draw();
    }
    private void buttonCreateShipClick(){
        Random random = new Random();
        pictureBox.setBounds(0,0,getContentPane().getWidth(),getContentPane().getHeight());
        drawingAircraft = new DrawingAircraft(random.nextInt(200) + 100, random.nextInt(2000) + 1000, new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)),
                pictureBox.getWidth(), pictureBox.getHeight(), random.nextInt(3),(random.nextInt(3)+1)*2);
        drawingAircraft.setPosition(random.nextInt(90) + 10, random.nextInt(90) + 10);
        draw();
    }
    private void buttonStepClick() {
        if (drawingAircraft == null) {
            return;
        }
        if (comboBoxStrategy.isEnabled()) {
            switch(comboBoxStrategy.getSelectedIndex()) {
                case 0:
                    abstractStrategy = new MoveToCenter();
                    break;
                case 1:
                    abstractStrategy = new MoveToBorder();
                    break;
                default:
                    abstractStrategy = null;
                    break;
            };
            if (abstractStrategy == null)
            {
                return;
            }
            abstractStrategy.SetData(new DrawingObjectAircraftCarrier(drawingAircraft), pictureBox.getWidth(),
                    pictureBox.getHeight());
            comboBoxStrategy.setEnabled(false);
        }
        if (abstractStrategy == null)
        {
            return;
        }
        abstractStrategy.MakeStep();
        draw();
        if (abstractStrategy.GetStatus() == Status.FINISH)
        {
            comboBoxStrategy.setEnabled(true);
            abstractStrategy = null;
        }
    }
    private void buttonMoveClick(ActionEvent event) {
        if(drawingAircraft == null || drawingAircraft.getEntityShip() == null)
            return;
        switch (event.getActionCommand())
        {
            case "left":
                drawingAircraft.moveTransport(DirectionType.LEFT);
                break;
            case "right":
                drawingAircraft.moveTransport(DirectionType.RIGHT);
                break;
            case "up":
                drawingAircraft.moveTransport(DirectionType.UP);
                break;
            case "down":
                drawingAircraft.moveTransport(DirectionType.DOWN);
                break;
        }
        draw();
    }
    private void draw() {
        if (drawingAircraft == null)
            return;
        pictureBox.repaint();
    }
}